/* main.tsx - Ultra-optimized for minimal HTTP requests */

// Import the bootstrap script first for immediate execution
import "./bootstrap.js";

// Import only what's absolutely necessary for first render
import { createRoot } from "react-dom/client";
import App from "./App";

// CRITICAL OPTIMIZATION: Only import CSS through JS for the homepage
// This is crucial for reducing HTTP requests - we'll handle other CSS differently
import "./index.css";

// Import performance optimization utilities
import { initPerformanceMonitoring, reportPerformanceMetrics, deferOperation } from './utils/script-optimizer';
import { setupLazyLoading } from './utils/image-optimizer';

// Global performance tracking
if (typeof window !== 'undefined' && window.performance) {
  window.performance.mark('js-execution-start');
  // Initialize performance monitoring
  deferOperation(() => {
    initPerformanceMonitoring();
    setupLazyLoading();
  }, 'high');
}

// Resource loading optimization
const resourceLoader = {
  // Track loaded resources to avoid duplicates
  loadedResources: new Set<string>(),
  
  // Load CSS with high priority
  loadCSS: (href: string, priority = 'low') => {
    if (resourceLoader.loadedResources.has(href)) return;
    
    const link = document.createElement('link');
    link.rel = 'stylesheet';
    link.href = href;
    link.setAttribute('fetchpriority', priority);
    document.head.appendChild(link);
    resourceLoader.loadedResources.add(href);
  },
  
  // Prefetch future routes
  prefetchRoute: (route: string) => {
    if (resourceLoader.loadedResources.has(route)) return;
    
    const link = document.createElement('link');
    link.rel = 'prefetch';
    link.href = route;
    document.head.appendChild(link);
    resourceLoader.loadedResources.add(route);
  },
  
  // Preload critical images
  preloadImage: (src: string, priority = 'low') => {
    if (resourceLoader.loadedResources.has(src)) return;
    
    const link = document.createElement('link');
    link.rel = 'preload';
    link.as = 'image';
    link.href = src;
    link.setAttribute('fetchpriority', priority);
    document.head.appendChild(link);
    resourceLoader.loadedResources.add(src);
  }
};

// Immediate mounting of critical UI elements
// This ensures the fastest possible rendering of the initial view
const root = document.getElementById("root")!;
// Use createRoot synchronously to get the app on screen as quickly as possible
const reactRoot = createRoot(root);

// Hyper-optimized application bootstrap with minimal HTTP requests
const bootstrapApp = () => {
  // PHASE 1: Start React rendering immediately
  reactRoot.render(<App />);
  
  // PHASE 2: Optimize resources only after initial render
  if (typeof window !== 'undefined') {
    // Resource loading optimization
    const optimizeResources = () => {
      try {
        // 1. Remove the placeholder content 
        const placeholder = document.querySelector('.initial-placeholder');
        if (placeholder && placeholder.parentNode) {
          placeholder.parentNode.removeChild(placeholder);
        }
        
        // 2. Lazy-load below-the-fold images using IntersectionObserver
        const lazyImages = document.querySelectorAll('img[loading="lazy"]');
        if ('IntersectionObserver' in window) {
          const imageObserver = new IntersectionObserver((entries) => {
            entries.forEach(entry => {
              if (entry.isIntersecting) {
                const img = entry.target as HTMLImageElement;
                if (img.dataset.src) {
                  img.src = img.dataset.src;
                  imageObserver.unobserve(img);
                }
              }
            });
          }, {
            rootMargin: '200px' // Load images 200px before they enter viewport
          });
          
          lazyImages.forEach(img => {
            imageObserver.observe(img);
          });
        }
        
        // 3. Conditionally prefetch top routes
        const shouldPrefetch = !('connection' in navigator) || 
          !(navigator as any).connection?.saveData;
          
        if (shouldPrefetch) {
          // Only prefetch for users with good connections
          const criticalRoutes = ['/food-intolerance', '/contact'];
          criticalRoutes.forEach(route => resourceLoader.prefetchRoute(route));
        }
        
        // 4. Optimize CSS delivery
        const criticalStyles = document.querySelector('style[data-critical="true"]');
        if (criticalStyles) {
          // Mark as non-blocking after content is visible
          setTimeout(() => {
            criticalStyles.setAttribute('media', 'all');
          }, 2000);
        }
        
        // Load deferred non-critical CSS
        const loadDeferredCss = () => {
          const link = document.createElement('link');
          link.rel = 'stylesheet';
          link.href = '/src/index.css.defer';
          link.setAttribute('media', 'print');
          link.setAttribute('onload', "this.media='all'");
          document.head.appendChild(link);
        };
        
        // Load deferred CSS after the main content is interactive
        setTimeout(loadDeferredCss, 1000);
        
        // 5. Preload logo and key images for better UX during navigation
        resourceLoader.preloadImage('/images/logo.webp', 'high');
      } catch (err) {
        // Silent fail - non-critical
        console.error('Resource optimization error:', err);
      }
    };
    
    // Execute resource optimization after first meaningful paint
    if (document.readyState === 'complete') {
      optimizeResources();
    } else {
      window.addEventListener('load', optimizeResources);
    }
    
    // Track and report performance metrics
    setTimeout(() => {
      if (window.performance && window.performance.mark) {
        window.performance.mark('app-interactive');
        window.performance.measure(
          'time-to-interactive', 
          'js-execution-start', 
          'app-interactive'
        );
      }
    }, 500);
  }
};

// Start rendering as fast as humanly possible
bootstrapApp();
